@@ -1,68 +1,64 @@
-
-# handled by Anirudh
-
 from json import loads
+from .i18n import i18n_translate
+from . import gs_global
 
-
-def show_response_card(gs_context, current_context, response_details):
+def show_response_card(gs_context,current_context,response_details):
     channel = gs_context['channel']
+
+    i18n_required = gs_global.global_context['i18n_required']
+    print('15451',i18n_required,gs_context["slots"]['language'],gs_context['menu_handled'])
+    if(i18n_required): #and gs_context["menu_handled"] == False:
+        response_details = i18n_translate(response_details,gs_context["slots"]['language'],channel)
     if response_details['card_type'] == 'list':
-        validate_list(channel, response_details)
-        list_json = render_list(channel, response_details)
+        validate_list(channel,response_details)
+        list_json = render_list(channel,response_details)
         current_context['response_queue'].append(list_json)
     elif response_details['card_type'] == 'quick_reply':
-        validate_quick_reply(channel, response_details)
-        quick_reply_json = render_quick_reply(channel, response_details)
+        validate_quick_reply(channel,response_details)
+        quick_reply_json = render_quick_reply(channel,response_details)
         current_context['response_queue'].append(quick_reply_json)
     elif response_details['card_type'] == 'text':
         text = response_details['message']
         current_context['response_queue'].append(str(text))
     elif response_details['card_type'] == 'file':
-        file_json = render_file(channel, response_details)
-        current_context['response_queue'].append(file_json)
+        file_json = render_file(channel,response_details)
+        current_context['response_queue'].append(file_json) 
     elif response_details['card_type'] == 'image':
-        img_json = render_image(channel, response_details)
+        img_json = render_image(channel,response_details)
         current_context['response_queue'].append(img_json)
     elif response_details['card_type'] == 'catalogue':
-        catalogue_json = render_catalogue(channel, response_details)
+        catalogue_json = render_catalogue(channel,response_details)
         current_context['response_queue'].append(catalogue_json)
     elif response_details['card_type'] == 'generic_card':
-        generic_card_json = render_generic_card(channel, response_details)
+        generic_card_json = render_generic_card(channel,response_details)
         current_context['response_queue'].append(generic_card_json)
     return
 
-
-def validate_list(channel, response_details):
+def validate_list(channel,response_details):
     if channel == 'whatsapp':
         len_title = len(response_details['main_title'])
         if len_title > 60:
-            raise Exception(
-                "main_title has exceeded the character limit of 60!")
+            raise Exception("main_title has exceeded the character limit of 60!")
         len_body = len(response_details['main_body'])
         if len_body > 1024:
-            raise Exception(
-                "main_body has exceeded the character limit of 1024!")
+            raise Exception("main_body has exceeded the character limit of 1024!")
         len_button = len(response_details['button_text'])
         if len_button > 20:
-            raise Exception(
-                "button_text has exceeded the character limit of 20!")
+            raise Exception("button_text has exceeded the character limit of 20!")
         for section in response_details['sections']:
             if 'title' not in section.keys():
                 raise Exception("Section does not have a Title!")
             for option in section['options']:
                 len_opt_title = len(option['title'])
                 if len_opt_title > 24:
-                    raise Exception(
-                        f"'{option['title']}' has exceeded the character limit of 24!")
+                    raise Exception(f"'{option['title']}' has exceeded the character limit of 24!")
                 if 'description' in option.keys():
                     len_desc = len(option['description'])
                     if len_desc > 72:
-                        raise Exception(
-                            f"'{option['description']}' has exceeded the character limit of 72!")
+                        raise Exception(f"'{option['description']}' has exceeded the character limit of 72!")
     return
 
-
-def render_list(channel, response_details):
+def render_list(channel,response_details):
     list_json = {}
     if channel == 'whatsapp':
         main_title = response_details['main_title']
@@ -91,8 +87,7 @@ def render_list(channel, response_details):
         list_json['body'] = main_body
         list_json['msgid'] = 'lst1'
         list_json['globalButtons'] = []
-        list_json['globalButtons'].append(
-            {"type": "text", "title": str(button_text)})
+        list_json['globalButtons'].append({"type":"text","title":str(button_text)})
         list_json['items'] = item_list
     elif channel == 'gip':
         list_json['type'] = 'list'
@@ -108,27 +103,22 @@ def render_list(channel, response_details):
         list_json['globalButtons'] = response_details['globalButtons']
     return list_json
 
-
-def validate_quick_reply(channel, response_details):
+def validate_quick_reply(channel,response_details):
     if channel == 'whatsapp':
         if 'header' in response_details.keys():
             len_header = len(response_details['header'])
             if len_header > 60:
-                raise Exception(
-                    "header has exceeded the character limit of 60!")
+                raise Exception("header has exceeded the character limit of 60!")
         if 'caption' in response_details.keys():
             len_caption = len(response_details['caption'])
             if len_caption > 60:
-                raise Exception(
-                    "cation has exceeded the character limit of 60!")
+                raise Exception("cation has exceeded the character limit of 60!")
         len_options = len(response_details['options'])
         if len_options > 3:
-            raise Exception(
-                "You cannot use more than 3 quick reply buttons in whatsapp!")
+            raise Exception("You cannot use more than 3 quick reply buttons in whatsapp!")
     return
 
-
-def render_quick_reply(channel, response_details):
+def render_quick_reply(channel,response_details):
     quick_reply_json = {}
     if channel == 'whatsapp':
         buttons = response_details['options']
@@ -170,10 +160,17 @@ def render_quick_reply(channel, response_details):
                 button_dict['iconurl'] = button['url']
             options_list.append(button_dict)
         quick_reply_json['options'] = options_list
+    elif channel == 'instagram':
+        quick_reply_json['type'] = 'quick_reply'
+        quick_reply_json["msgid"] = "qr1"
+        content = {}
+        content['type'] = 'text'
+        content['text'] = response_details['text']
+        quick_reply_json['content'] = content
+        quick_reply_json['options'] = response_details['options']
     return quick_reply_json
 
-
-def render_file(channel, response_details):
+def render_file(channel,response_details):
     file_json = {}
     if channel == 'whatsapp':
         file_json['type'] = 'file'
@@ -182,11 +179,10 @@ def render_file(channel, response_details):
     elif channel == 'gip':
         file_json['type'] = 'file'
         file_json['url'] = response_details['url']
-        file_json['text'] = response_details['text']
+        file_json['text'] = response_details['text']       
     return file_json
 
-
-def render_image(channel, response_details):
+def render_image(channel,response_details):
     image_json = {}
     if channel == 'whatsapp':
         image_json['type'] = 'image'
@@ -197,10 +193,13 @@ def render_image(channel, response_details):
         image_json['type'] = 'image'
         image_json['originalUrl'] = response_details['url']
         image_json['previewUrl'] = response_details['url']
+    elif channel == 'instagram':
+        image_json['type'] = 'image'
+        image_json['originalUrl'] = response_details['url']
+        image_json['previewUrl'] = response_details['url']             
     return image_json
 
-
-def render_generic_card(channel, response_details):
+def render_generic_card(channel,response_details):
     generic_json = {}
     if channel == 'whatsapp':
         raise Exception('Generic Cards are not supported in Whatsapp!')
@@ -208,21 +207,22 @@ def render_generic_card(channel, response_details):
         generic_json['type'] = 'generic_card'
         generic_json['mml'] = {}
         generic_json['mml'] = response_details
-        generic_json['mml']['type'] = 'generic_card'
+        generic_json['mml']['type'] = 'generic_card'        
     return generic_json
 
-
-def render_catalogue(channel, response_details):
+def render_catalogue(channel,response_details):
     catalogue_json = {}
     if channel == 'whatsapp':
         raise Exception('Catalogues are not supported in Whatsapp yet!')
     elif channel == 'gip':
         catalogue_json = response_details
         catalogue_json['type'] = 'catalogue'
-    return catalogue_json
+    elif channel == 'instagram':
+        catalogue_json = response_details
+        catalogue_json['type'] = 'catalogue'      
+    return catalogue_json    
 
-
-def behave_extract_bot_response_text(channel, bot_response):
+def behave_extract_bot_response_text(channel,bot_response):
     if channel == 'whatsapp':
         bot_text = ''
         if len(bot_response) > 1:
@@ -243,15 +243,23 @@ def behave_extract_bot_response_text(channel, bot_response):
         else:
             bot_text = bot_response[0]['text']
         return bot_text
+    if channel == 'instagram':
+        bot_text = ''
+        if len(bot_response) > 1:
+            for resp in bot_response:
+                if resp['text'][0] != '{':
+                    bot_text = resp['text']
+                    break
+        else:
+            bot_text = bot_response[0]['text']
+        return bot_text
 
-
-def behave_compare_text(channel, input_text, bot_text):
+def behave_compare_text(channel,input_text,bot_text):
     if input_text == bot_text:
         return True
     return False
 
-
-def behave_extract_bot_response_quick_reply(channel, bot_response):
+def behave_extract_bot_response_quick_reply(channel,bot_response):
     if channel == 'whatsapp':
         bot_text = ''
         bot_buttons = []
@@ -261,14 +269,14 @@ def behave_extract_bot_response_quick_reply(channel, bot_response):
                     qr_json = loads(resp['text'])
                     bot_text = qr_json['content']['text']
                     for option in qr_json['options']:
-                        bot_buttons.append(option['title'])
+                        bot_buttons.append(option['title'])  
                     break
         else:
             qr_json = loads(bot_response[0]['text'])
             bot_text = qr_json['content']['text']
             for option in qr_json['options']:
                 bot_buttons.append(option['title'])
-        return (bot_text.strip(), bot_buttons.sort())
+        return (bot_text.strip(),bot_buttons.sort())
     if channel == 'gip':
         bot_content = ''
         bot_buttons = []
@@ -281,7 +289,7 @@ def behave_extract_bot_response_quick_reply(channel, bot_response):
                     else:
                         bot_content = qr_json['content']['url']
                     for option in qr_json['options']:
-                        bot_buttons.append(option)
+                        bot_buttons.append(option)  
                     break
         else:
             qr_json = loads(bot_response[0]['text'])
@@ -291,11 +299,27 @@ def behave_extract_bot_response_quick_reply(channel, bot_response):
                 bot_content = qr_json['content']['url']
             for option in qr_json['options']:
                 del option['type']
+                bot_buttons.append(option) 
+        return (bot_content.strip(),bot_buttons)
+    if channel == 'instagram':
+        bot_text = ''
+        bot_buttons = []
+        if len(bot_response) > 1:
+            for resp in bot_response:
+                if resp['text'][0] == '{':
+                    qr_json = loads(resp['text'])
+                    bot_text = qr_json['content']['text']
+                    for option in qr_json['options']:
+                        bot_buttons.append(option['title'])  
+                    break
+        else:
+            qr_json = loads(bot_response[0]['text'])
+            bot_text = qr_json['content']['text']
+            for option in qr_json['options']:
                 bot_buttons.append(option)
-        return (bot_content.strip(), bot_buttons)
+        return (bot_text.strip(),bot_buttons.sort()) 
 
-
-def behave_compare_quick_reply(channel, input_content, bot_resp):
+def behave_compare_quick_reply(channel,input_content,bot_resp):
     if channel == 'whatsapp':
         input_message = input_content['message'].strip()
         button_input = input_content['buttons'].sort()
@@ -328,9 +352,18 @@ def behave_compare_quick_reply(channel, input_content, bot_resp):
             if button not in bot_buttons:
                 return False, "Buttons did not match!"
         return True, None
+    elif channel == 'instagram':
+        input_message = input_content['message'].strip()
+        button_input = input_content['buttons'].sort()
+        bot_message = bot_resp[0]
+        bot_buttons = bot_resp[1]
+        if (input_message != bot_message):
+            return False, "Message comparison failed!"
+        if (button_input != bot_buttons):
+            return False, "Buttons did not match!"
+        return True, None
 
-
-def behave_extract_bot_response_list(channel, bot_response):
+def behave_extract_bot_response_list(channel,bot_response):
     if channel == 'whatsapp':
         bot_text = ''
         bot_list = []
@@ -357,7 +390,7 @@ def behave_extract_bot_response_list(channel, bot_response):
                 for option in item['options']:
                     list_element["Items"].append(option['title'])
                 bot_list.append(list_element)
-        return (bot_text.strip(), bot_list)
+        return (bot_text.strip(),bot_list)
     elif channel == 'gip':
         bot_items = []
         bot_buttons = []
@@ -375,11 +408,10 @@ def behave_extract_bot_response_list(channel, bot_response):
             for item in list_json['items']:
                 bot_items.append(item)
             for option in list_json['globalButtons']:
-                bot_buttons.append(option)
-        return (bot_items, bot_buttons)
+                bot_buttons.append(option)           
+        return (bot_items,bot_buttons)  
 
-
-def behave_compare_list(channel, input_content, bot_resp):
+def behave_compare_list(channel,input_content,bot_resp):
     if channel == 'whatsapp':
         input_message = input_content['message'].strip()
         list_input = input_content['list']
@@ -443,21 +475,19 @@ def behave_compare_list(channel, input_content, bot_resp):
             return False, "Button Comaparison Failed!"
         return True, None
 
-
-def behave_extract_bot_response_file(channel, bot_response):
+def behave_extract_bot_response_file(channel,bot_response):
     if channel == 'whatsapp':
         file_json = loads(bot_response[0]['text'])
         bot_filename = file_json['filename']
         bot_url = file_json['url']
-        return (bot_filename, bot_url)
+        return (bot_filename,bot_url)
     elif channel == 'gip':
         file_json = loads(bot_response[0]['text'])
         bot_text = file_json['text']
         bot_url = file_json['url']
-        return (bot_text, bot_url)
+        return (bot_text,bot_url)
 
-
-def behave_compare_file(channel, input_content, bot_resp):
+def behave_compare_file(channel,input_content,bot_resp):
     if channel == 'whatsapp':
         input_filename = input_content['filename']
         input_url = input_content['url']
@@ -473,26 +503,28 @@ def behave_compare_file(channel, input_content, bot_resp):
         input_url = input_content['url']
         bot_filename = bot_resp[0]
         bot_url = bot_resp[1]
-        if (input_text != bot_filename):
+        if (input_text != bot_filename): 
             return False, "Text did not match!"
         if (input_url != bot_url):
             return False, "URLs did not match!"
-        return True, None
+        return True, None       
 
-
-def behave_extract_bot_response_image(channel, bot_response):
+def behave_extract_bot_response_image(channel,bot_response):
     if channel == 'whatsapp':
         image_json = loads(bot_response[0]['text'])
         bot_caption = image_json['caption']
         bot_url = image_json['originalUrl']
-        return (bot_caption, bot_url)
+        return (bot_caption,bot_url)
     elif channel == 'gip':
         image_json = loads(bot_response[0]['text'])
         bot_url = image_json['originalUrl']
-        return bot_url, None
+        return bot_url,None
+    elif channel == 'instagram':
+        image_json = loads(bot_response[0]['text'])
+        bot_url = image_json['originalUrl']
+        return bot_url,None
 
-
-def behave_compare_image(channel, input_content, bot_resp):
+def behave_compare_image(channel,input_content,bot_resp):
     if channel == 'whatsapp':
         input_url = input_content['url']
         input_caption = input_content['caption']
@@ -509,9 +541,14 @@ def behave_compare_image(channel, input_content, bot_resp):
         if (input_url != bot_url):
             return False, "URLs did not match!"
         return True, None
+    elif channel == 'instagram':
+        input_url = input_content['url']
+        bot_url = bot_resp[0]
+        if (input_url != bot_url):
+            return False, "URLs did not match!"
+        return True, None
 
-
-def behave_extract_bot_response_catalogue(channel, bot_response):
+def behave_extract_bot_response_catalogue(channel,bot_response):
     if channel == 'whatsapp':
         raise Exception('Catalogue not supported in Whatsapp yet!')
     elif channel == 'gip':
@@ -526,11 +563,24 @@ def behave_extract_bot_response_catalogue(channel, bot_response):
         else:
             cat_json = loads(bot_response[0]['text'])
             for item in cat_json['items']:
-                bot_items.append(item)
+                bot_items.append(item)           
         return bot_items
+    elif channel == 'instagram':
+        bot_items = []
+        if len(bot_response) > 1:
+            for resp in bot_response:
+                if resp['text'][0] == '{':
+                    cat_json = loads(resp['text'])
+                    for item in cat_json['items']:
+                        bot_items.append(item)
+                    break
+        else:
+            cat_json = loads(bot_response[0]['text'])
+            for item in cat_json['items']:
+                bot_items.append(item)           
+        return bot_items    
 
-
-def behave_compare_catalogue(channel, input_content, bot_resp):
+def behave_compare_catalogue(channel,input_content,bot_resp):
     if channel == 'whatsapp':
         raise Exception('Catalogue not supported in Whatsapp yet!')
     if channel == 'gip':
@@ -568,9 +618,45 @@ def behave_compare_catalogue(channel, input_content, bot_resp):
                 fstr = f"Item with title {title} did not match!"
                 return False, fstr
         return True, None
+    if channel == 'instagram':
+        input_items = input_content['items']
+        bot_items = bot_resp
+        default_actions_removed = []
+        # remove N.A. default actions and webview_height_ratio in options
+        for item in input_items:
+            item['title'] = item['title'].encode().decode('unicode_escape')
+            item['subtitle'] = item['subtitle'].encode().decode('unicode_escape')
+            item['options'] = item.pop('buttons')
+            item['options'] = sorted(item['options'], key=lambda d: d['title'])
+            if 'url' in item.keys():
+                item['imgurl'] = item.pop('url')
+            if 'defaultAction' in item.keys():
+                if item['defaultAction'] == 'N.A.':
+                    del item['defaultAction']
+                    default_actions_removed.append(item['title'])
+                else:
+                    item['defaultaction'] = item.pop('defaultAction')
+            else:
+                default_actions_removed.append(item['title'])
+        for item in bot_items:
+            item['title'] = item['title'].encode().decode('unicode_escape')
+            item['subtitle'] = item['subtitle'].encode().decode('unicode_escape')
+            item['options'] = sorted(item['options'], key=lambda d: d['title'])
+            if item['title'] in default_actions_removed:
+                if 'defaultaction' in item.keys():
+                    del item['defaultaction']
+            for option in item['options']:
+                if 'webview_height_ratio' in option.keys():
+                    del option['webview_height_ratio']
+        # compare items
+        for item in input_items:
+            if item not in bot_items:
+                title = item['title']
+                fstr = f"Item with title {title} did not match!"
+                return False, fstr
+        return True, None
 
-
-def behave_extract_bot_response_generic_card(channel, bot_response):
+def behave_extract_bot_response_generic_card(channel,bot_response):
     if channel == 'whatsapp':
         raise Exception("Generic Cards are not supported on Whatsapp!")
     elif channel == 'gip':
@@ -583,17 +669,16 @@ def behave_extract_bot_response_generic_card(channel, bot_response):
                     break
         else:
             gencard = loads(bot_response[0]['text'])
-            bot_dict = gencard['mml']
+            bot_dict = gencard['mml']           
         return bot_dict
 
-
-def behave_compare_generic_card(channel, input_content, bot_resp):
+def behave_compare_generic_card(channel,input_content,bot_resp):
     if channel == 'whatsapp':
         raise Exception("Generic Cards are not supported on Whatsapp!")
     elif channel == 'gip':
         gencard_input = {}
         gencard_input['message'] = input_content['message']
-        i = 0
+        i=0
         for title in input_content['titles']:
             key_str = 'title_' + str(i)
             gencard_input[key_str] = title
@@ -605,15 +690,14 @@ def behave_compare_generic_card(channel, input_content, bot_resp):
             else:
                 gencard_input['core_entity'] = input_content['core_entity']['text']
                 gencard_input['action0_rank'] = 0
-                gencard_input['action0_type'] = int(
-                    input_content['core_entity']['type'])
+                gencard_input['action0_type'] = int(input_content['core_entity']['type'])
                 gencard_input['action0_to'] = input_content['core_entity']['url']
                 if gencard_input['action0_type'] == 4:
                     gencard_input['action0_content'] = gencard_input['core_entity']
                     gencard_input['action0_to'] = gencard_input['core_entity']
                 else:
                     gencard_input['action0_content'] = gencard_input['action0_to']
-        j = 1
+        j=1
         for action in input_content['actions']:
             key_rank = 'action' + str(j) + '_rank'
             key_type = 'action' + str(j) + '_type'
@@ -623,8 +707,8 @@ def behave_compare_generic_card(channel, input_content, bot_resp):
             gencard_input[key_type] = action['type']
             gencard_input[key_to] = action['to']
             gencard_input[key_content] = action['content']
-            j = j+1
-        for key, value in gencard_input.items():
+            j=j+1
+        for key,value in gencard_input.items():
             if key not in bot_resp.keys():
                 reason = f"Key={key} not found in bot response!"
                 return False, reason
@@ -633,8 +717,7 @@ def behave_compare_generic_card(channel, input_content, bot_resp):
                 return False, reason
         return True, None
 
-
-def behave_extract_bot_response_generic_card_media(channel, bot_response):
+def behave_extract_bot_response_generic_card_media(channel,bot_response):
     if channel == 'whatsapp':
         raise Exception("Generic Cards are not supported on Whatsapp!")
     elif channel == 'gip':
@@ -647,17 +730,16 @@ def behave_extract_bot_response_generic_card_media(channel, bot_response):
                     break
         else:
             gencard = loads(bot_response[0]['text'])
-            bot_dict = gencard['mml']
+            bot_dict = gencard['mml']           
         return bot_dict
 
-
-def behave_compare_generic_card_media(channel, input_content, bot_resp):
+def behave_compare_generic_card_media(channel,input_content,bot_resp):
     if channel == 'whatsapp':
         raise Exception("Generic Cards are not supported on Whatsapp!")
     elif channel == 'gip':
         gencard_input = {}
         gencard_input['message'] = input_content['message']
-        i = 0
+        i=0
         for title in input_content['titles']:
             key_str = 'title_' + str(i)
             gencard_input[key_str] = title
@@ -667,12 +749,11 @@ def behave_compare_generic_card_media(channel, input_content, bot_resp):
                 if input_content['media'] != 'N.A.':
                     gencard_input['media'] = input_content['media']
             else:
-                gencard_input['media1_type'] = int(
-                    input_content['media']['type'])
+                gencard_input['media1_type'] = int(input_content['media']['type'])
                 gencard_input['media1_url'] = input_content['media']['url']
                 if gencard_input['media1_type'] != 1:
                     gencard_input['media1_cover'] = gencard_input['media']['cover']
-        j = 1
+        j=1
         for action in input_content['actions']:
             key_rank = 'action' + str(j) + '_rank'
             key_type = 'action' + str(j) + '_type'
@@ -682,8 +763,8 @@ def behave_compare_generic_card_media(channel, input_content, bot_resp):
             gencard_input[key_type] = action['type']
             gencard_input[key_to] = action['to']
             gencard_input[key_content] = action['content']
-            j = j+1
-        for key, value in gencard_input.items():
+            j=j+1
+        for key,value in gencard_input.items():
             if key not in bot_resp.keys():
                 reason = f"Key={key} not found in bot response!"
                 return False, reason
